==> heteroscalapack-1.0.2-BETA.tar.gz: heteroscalapack-1.0.2-BETA/SRC/hscal_pzgeqrf_info.c <==

#include <math.h>
#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>

#include "../include/hscalapack.h"

/*------------------------------------------------------------------*/

double hscal_pzgeqrf_recon_tcomp(
   int M,
   int N
)
{
    double tcomp = 0.;

    char   transac  = 'N';
    char   *transa  = &transac;

    char   transbc  = 'C';
    char   *transb  = &transbc;

    int reconm, reconn, reconk;

    int num_of_processors = HMPI_Group_size(HMPI_PROC_WORLD_GROUP);

    int    lda, ldb, ldc;

    double alpha1[CNARG], alpha2[CNARG];

    alpha1[REAL_PART] = -1.;
    alpha1[IMAG_PART] = -0.;

    alpha2[REAL_PART] = 1.;
    alpha2[IMAG_PART] = 0.;

    reconm  = M/sqrt(num_of_processors);
    reconn  = N/sqrt(num_of_processors);
    reconk  = hscal_pilaenv_recon("C");

    lda = reconm;
    ldb = reconk;
    ldc = reconm;

    hscal_zgemm(
         transa,
         transb,
         &reconm,
         &reconn,
         &reconk,
         ((char*)alpha1),
         &lda,
         &ldb,
         ((char*)alpha2),
         &ldc,
         &tcomp
    );

    HSCAL_DEBUG(
         __FILE__, __FUNCTION__,__LINE__, 
         "Number of recon computations %f",
         tcomp
    );

    return tcomp;
}

/*------------------------------------------------------------------*/

double hscal_pzgeqrf_my_tcomp(
   int myrow,
   int mycol,
   int M,
   int N,
   int IA,
   int JA,
   int DESCA[DLEN1_],
   int LWORK,
   int nprow,
   int npcol
)
{
   int rc;
   double myt, *tcomp = (double*)calloc(nprow*npcol, sizeof(double));
   if (tcomp == NULL)
   {
      return HSCAL_ERR_LOW_MEMORY;
   }

   rc = hscal_pzgeqrf_my_tcomp_tcomm
        (
          1,
          myrow, mycol,
          M, N,
          IA, JA, DESCA, LWORK,
          nprow, npcol,
          tcomp, NULL
    );

    if (rc != HSCAL_SUCCESS)
    {
       return rc;
    }

    HSCAL_DEBUG(
         __FILE__, __FUNCTION__,__LINE__, 
         "Number of my computations %f", 
         TCOMP(myrow, mycol, nprow, npcol)
    );

    myt = TCOMP(myrow, mycol, nprow, npcol);
    free(tcomp);
    return myt;
}

/*------------------------------------------------------------------*/

int hscal_pzgeqrf_my_tcomm(
   int myrow,
   int mycol,
   int M,
   int N,
   int IA,
   int JA,
   int DESCA[DLEN1_],
   int LWORK,
   int nprow,
   int npcol,
   int *tcomm
)
{
   int i, j, rc;

   int *tcomm_l = (int*)calloc(
                        nprow*npcol*nprow*npcol, 
                        sizeof(int)
   );

   if (tcomm_l == NULL)
   {
      return HSCAL_ERR_LOW_MEMORY;
   }

   rc = hscal_pzgeqrf_my_tcomp_tcomm
        (
          1,
          myrow, mycol,
          M, N,
          IA, JA, DESCA, LWORK,
          nprow, npcol,
          NULL, tcomm_l
   );

   if (rc != HSCAL_SUCCESS)
   {
      return rc;
   }

   for (i = 0; i < nprow; i++)
   {
      for (j = 0; j < npcol; j++)
      {
          tcomm[HSCALRIX(i, j, nprow, npcol)] 
          = 
          TCOMML(myrow, mycol, i, j, nprow, npcol);
      }
   }

   free(tcomm_l);

   return HSCAL_SUCCESS;
}

/*------------------------------------------------------------------*/

int hscal_pzgeqrf_my_tcomm_to_ij(
   int myrow,
   int mycol,
   int ireceiver,
   int jreceiver,
   int M,
   int N,
   int IA,
   int JA,
   int DESCA[DLEN1_],
   int LWORK,
   int nprow,
   int npcol,
   int *tcomm,
   int *tcomm_i
)
{
   int i, j, rc;

   if (TCOMMI(myrow, mycol, nprow, npcol)
       != HSCAL_COMM_UNINITIALIZED)
   {
      return TCOMM(myrow, mycol, ireceiver, jreceiver, nprow, npcol);
   }

   TCOMMI(myrow, mycol, nprow, npcol) = 1;

   for (i = 0; i < nprow; i++)
   {
      for (j = 0; j < npcol; j++)
      {
          TCOMM(myrow, mycol, i, j, nprow, npcol) = 0;
      }
   }

   rc = hscal_pzgeqrf_my_tcomp_tcomm
        (
          1,
          myrow, mycol,
          M, N,
          IA, JA, DESCA, LWORK,
          nprow, npcol,
          NULL, tcomm
    );

    if (rc != HSCAL_SUCCESS)
    {
       return rc;
    }

    HSCAL_DEBUG(
         __FILE__, __FUNCTION__,__LINE__,
         "(%d, %d) -> (%d, %d), (p, q)=(%d, %d) %d",
         myrow, mycol, ireceiver, jreceiver, nprow, npcol,
         TCOMM(myrow, mycol, ireceiver, jreceiver, nprow, npcol)
    );

    return TCOMM(myrow, mycol, ireceiver, jreceiver, nprow, npcol);
}

/*------------------------------------------------------------------*/

int hscal_pzgeqrf_my_tcomp_tcomm(
   int init_llds_f,
   int myrow,
   int mycol,
   int M,
   int N,
   int IA,
   int JA,
   int DESCA[DLEN1_],
   int LWORK,
   int nprow,
   int npcol,
   double *tcomp,
   int *tcomm
)
{
    hscalinteger c__1 = 1;
    hscalinteger c__2 = 2;
    hscalinteger c__6 = 6;

    /* System generated locals */
    hscalinteger i__1, i__2, i__3, i__4, i__5;
    hscaldoublereal d__1;
    hscaldoublecomplex z__1;

    /* Local variables */
    hscalinteger i__, j, k;
    hscalinteger jb, jn, mp0, nq0, ipw, idum1[1], idum2[1];
    hscalinteger icoff, iacol, iinfo, iarow, lwmin, ictxt;
    hscallogical lquery;
    char colbtop[1], rowbtop[1];

    int     *m      = &M;
    int     *n      = &N;
    int     *ia     = &IA;
    int     *ja     = &JA;
    int     *desca  = (int*)DESCA;
    int     *lwork  = &LWORK;
    int     info_l, *info = &info_l;

    double  * tcomp_l = tcomp;
    int     * tcomm_l = tcomm;

    /* Parameter adjustments */
    --desca;

    if (init_llds_f == 1)
    {
       HSCAL_PXGEQRF_INIT_LLDS(
            (desca+1),
            myrow, nprow
       );
    }

    HSCAL_DEBUG(
        __FILE__, __FUNCTION__,__LINE__, 
        "PZGEQRF: (%d, %d): (desca+1)[M1_]=%d (desca+1)[MB1_]=%d myrow=%d (desca+1)[RSRC1_]=%d nprow=%d (desca+1)[LLD1_]=%d",
        myrow, mycol, (desca+1)[M1_], (desca+1)[MB1_], myrow, (desca+1)[RSRC1_], nprow, (desca+1)[LLD1_]
    );

/*     Test the input parameters */
    *info = 0;
    if (nprow == -1) {
	*info = -602;
    } else {
	hscal_chk1mat(m, &c__1, n, &c__2, ia, ja, &desca[1], &c__6, info,
                    myrow, mycol, nprow, npcol
      );
	if (*info == 0) {
	    icoff = (*ja - 1) % desca[6];
	    iarow = indxg2p_(ia, &desca[5], &myrow, &desca[7], &nprow);
	    iacol = indxg2p_(ja, &desca[6], &mycol, &desca[8], &npcol);
	    i__1 = *m + (*ia - 1) % desca[5];
	    mp0 = numroc_(&i__1, &desca[5], &myrow, &iarow, &nprow);
	    i__1 = *n + icoff;
	    nq0 = numroc_(&i__1, &desca[6], &mycol, &iacol, &npcol);
	    lwmin = desca[6] * (mp0 + nq0 + desca[6]);

          d__1 = (hscaldoublereal) lwmin;
          z__1.r = d__1, z__1.i = 0.;
	    lquery = *lwork == -1;
	    if (*lwork < lwmin && ! lquery) {
		*info = -9;
	    }
	}
	if (*lwork == -1) {
	    idum1[0] = -1;
	} else {
	    idum1[0] = 1;
	}
	idum2[0] = 9;
	hscal_pchk1mat(
           m, &c__1, n, &c__2, ia, ja, &desca[1], &c__6, &c__1, idum1, 
		idum2, info,
           &myrow, &mycol, &nprow, &npcol
      );
    }

    if (*info != 0) {
	i__1 = -(*info);
       hscal_PB_Cabort(
            "PZGEQRF",
            i__1,
            nprow, npcol, myrow, mycol
       );
       return HSCAL_SUCCESS;
    } else if (lquery) {
       return HSCAL_SUCCESS;
    }

/*     Quick return if possible */

    if (*m == 0 || *n == 0) {
	return HSCAL_SUCCESS;
    }

    k = min(*m,*n);
    ipw = desca[6] * desca[6] + 1;
    pb_topget__(&ictxt, "Broadcast", "Rowwise", rowbtop);
    pb_topget__(&ictxt, "Broadcast", "Columnwise", colbtop);
    pb_topset__(&ictxt, "Broadcast", "Rowwise", "I-ring");
    pb_topset__(&ictxt, "Broadcast", "Columnwise", " ");

/*     Handle the first block of columns separately */

/* Computing MIN */
    i__1 = iceil_(ja, &desca[6]) * desca[6], i__2 = *ja + k - 1;
    jn = min(i__1,i__2);
    jb = jn - *ja + 1;

/*     Compute the QR factorization of the first block A(ia:ia+m-1,ja:jn) */

    hscal_pzgeqr2_my_tcomp_tcomm(
         0,
         myrow, mycol,
         M, jb, IA, JA, &desca[1], LWORK,
         nprow, npcol,
         tcomp, tcomm
    );

    if (*ja + jb <= *ja + *n - 1) {

/*        Form the triangular factor of the block reflector */
/*        H = H(ja) H(ja+1) . . . H(jn) */

	hscal_pzlarft_my_tcomp_tcomm(
           0,
           myrow, mycol,
           'F', 'C', M, jb, IA, JA, &desca[1], 
           nprow, npcol,
           tcomp, tcomm
      );

/*        Apply H' to A(ia:ia+m-1,ja+jb:ja+n-1) from the left */

	i__1 = *n - jb;
	i__2 = *ja + jb;
	hscal_pzlarfb_my_tcomp_tcomm(
           0,
           myrow, mycol,
           'L', 'C', 'F', 'C', M, i__1, jb, IA, JA, &desca[1], 
           IA, i__2, &desca[1], 
           nprow, npcol,
           tcomp, tcomm
      );
    }

/*     Loop over the remaining blocks of columns */

    i__1 = *ja + k - 1;
    i__2 = desca[6];
    for (j = jn + 1; i__2 < 0 ? j >= i__1 : j <= i__1; j += i__2) {
/* Computing MIN */
	i__3 = k - j + *ja;
	jb = min(i__3,desca[6]);
	i__ = *ia + j - *ja;

/*        Compute the QR factorization of the current block */
/*        A(i:ia+m-1,j:j+jb-1) */

	i__3 = *m - j + *ja;
	hscal_pzgeqr2_my_tcomp_tcomm(
           0,
           myrow, mycol,
           i__3, jb, i__, j, &desca[1], 
	     LWORK,
           nprow, npcol,
           tcomp, tcomm
      );

	if (j + jb <= *ja + *n - 1) {

/*           Form the triangular factor of the block reflector */
/*           H = H(j) H(j+1) . . . H(j+jb-1) */

	    i__3 = *m - j + *ja;
	    hscal_pzlarft_my_tcomp_tcomm(
               0,
               myrow, mycol,
               'F', 'C', i__3, jb, i__, j, &desca[1],
               nprow, npcol,
               tcomp, tcomm
          );

/*           Apply H' to A(i:ia+m-1,j+jb:ja+n-1) from the left */

	    i__3 = *m - j + *ja;
	    i__4 = *n - j - jb + *ja;
	    i__5 = j + jb;
	    hscal_pzlarfb_my_tcomp_tcomm(
               0,
               myrow, mycol,
               'L', 'C', 'F', 'C', i__3, i__4, jb, i__, j, &desca[1],
		   i__, i__5, &desca[1],
               nprow, npcol,
               tcomp, tcomm
          );
	}

/* L10: */
    }

    pb_topset__(&ictxt, "Broadcast", "Rowwise", rowbtop);
    pb_topset__(&ictxt, "Broadcast", "Columnwise", colbtop);

   return HSCAL_SUCCESS;
}

/*------------------------------------------------------------------*/

int hscal_pzgeqrf_tcomp_tcomm(
   int M,
   int N,
   int IA,
   int JA,
   int DESCA[DLEN1_],
   int LWORK,
   int nprow,
   int npcol,
   double *tcomp,
   int *tcomm
)
{
   int i, rc, myrow, mycol;

   for (i = 0; i < nprow*npcol; i++)
   {
      tcomp[i] = 0;
   }

   for (i = 0; i < nprow*npcol*nprow*npcol; i++)
   {
      tcomm[i] = 0;
   }

   for (myrow = 0; myrow < nprow; myrow++)
   {
      for (mycol = 0; mycol < npcol; mycol++)
      {
         rc = hscal_pzgeqrf_my_tcomp_tcomm
              (
                1,
                myrow, mycol,
                M, N,
                IA, JA, DESCA, LWORK,
                nprow, npcol,
                tcomp, tcomm
         );

         if (rc != HSCAL_SUCCESS)
         {
            return rc;
         }
      }
   }

   return HSCAL_SUCCESS;
}

/*------------------------------------------------------------------*/

int hscal_pzgeqrf_info(
   int * M,
   int * N,
   int * IA,
   int * JA,
   int * DESCA,
   int * LWORK,
   int   nprow,
   int   npcol,
   double * tcomp,
   int    * tcomm
)
{
   int i, rc, myrow, mycol;

   for (i = 0; i < nprow*npcol; i++)
   {
      tcomp[i] = 0;
   }

   for (i = 0; i < nprow*npcol*nprow*npcol; i++)
   {
      tcomm[i] = 0;
   }

   for (myrow = 0; myrow < nprow; myrow++)
   {
      for (mycol = 0; mycol < npcol; mycol++)
      {
         rc = hscal_pzgeqrf_my_tcomp_tcomm
              (
                1,
                myrow, mycol,
                *M, *N,
                *IA, *JA, DESCA, *LWORK,
                nprow, npcol,
                tcomp, tcomm
         );

         if (rc != HSCAL_SUCCESS)
         {
            return rc;
         }
      }
   }

   return HSCAL_SUCCESS;
}

/*------------------------------------------------------------------*/

int hscal_pzgeqrf_info_i(
   int * M,
   int * N,
   int * IA,
   int * JA,
   int * DESCA,
   int * LWORK,
   int   nprow,
   int   npcol,
   double * tcomp,
   int    * tcomm
)
{
   int i, rc, myrow, mycol;

   for (myrow = 0; myrow < nprow; myrow++)
   {
      for (mycol = 0; mycol < npcol; mycol++)
      {
         rc = hscal_pzgeqrf_my_tcomp_tcomm
              (
                1,
                myrow, mycol,
                *M, *N,
                *IA, *JA, DESCA, *LWORK,
                nprow, npcol,
                tcomp, tcomm
         );

         if (rc != HSCAL_SUCCESS)
         {
            return rc;
         }
      }
   }

   return HSCAL_SUCCESS;
}

/*------------------------------------------------------------------*/

==> heteroscalapack-1.0.4-BETA.tar.gz: heteroscalapack-1.0.4-BETA/SRC/hscal_pzgeqrf_info.c <==

#include <math.h>
#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>

#include "../include/hscalapack.h"

/*------------------------------------------------------------------*/

double hscal_pzgeqrf_recon_tcomp(
   int M,
   int N
)
{
    double tcomp = 0.;

    char   transac  = 'N';
    char   *transa  = &transac;

    char   transbc  = 'C';
    char   *transb  = &transbc;

    int reconm, reconn, reconk;

    int num_of_processors = HMPI_Group_size(HMPI_PROC_WORLD_GROUP);

    int    lda, ldb, ldc;

    double alpha1[CNARG], alpha2[CNARG];

    alpha1[REAL_PART] = -1.;
    alpha1[IMAG_PART] = -0.;

    alpha2[REAL_PART] = 1.;
    alpha2[IMAG_PART] = 0.;

    reconm  = M/sqrt(num_of_processors);
    reconn  = N/sqrt(num_of_processors);
    reconk  = hscal_pilaenv_recon("C");

    lda = reconm;
    ldb = reconk;
    ldc = reconm;

    hscal_zgemm(
         transa,
         transb,
         &reconm,
         &reconn,
         &reconk,
         ((char*)alpha1),
         &lda,
         &ldb,
         ((char*)alpha2),
         &ldc,
         &tcomp
    );

    HSCAL_DEBUG(
         __FILE__, __FUNCTION__,__LINE__, 
         "Number of recon computations %f",
         tcomp
    );

    return tcomp;
}

/*------------------------------------------------------------------*/

double hscal_pzgeqrf_my_tcomp(
   int myrow,
   int mycol,
   int M,
   int N,
   int IA,
   int JA,
   int DESCA[DLEN1_],
   int nprow,
   int npcol
)
{
   int rc;
   double myt, *tcomp = (double*)calloc(nprow*npcol, sizeof(double));
   if (tcomp == NULL)
   {
      return HSCAL_ERR_LOW_MEMORY;
   }

   rc = hscal_pzgeqrf_my_tcomp_tcomm
        (
          1,
          myrow, mycol,
          M, N,
          IA, JA, DESCA,
          nprow, npcol,
          tcomp, NULL
    );

    if (rc != HSCAL_SUCCESS)
    {
       return rc;
    }

    HSCAL_DEBUG(
         __FILE__, __FUNCTION__,__LINE__, 
         "Number of my computations %f", 
         TCOMP(myrow, mycol, nprow, npcol)
    );

    myt = TCOMP(myrow, mycol, nprow, npcol);
    free(tcomp);
    return myt;
}

/*------------------------------------------------------------------*/

int hscal_pzgeqrf_my_tcomm(
   int myrow,
   int mycol,
   int M,
   int N,
   int IA,
   int JA,
   int DESCA[DLEN1_],
   int nprow,
   int npcol,
   int *tcomm
)
{
   int i, j, rc;

   int *tcomm_l = (int*)calloc(
                        nprow*npcol*nprow*npcol, 
                        sizeof(int)
   );

   if (tcomm_l == NULL)
   {
      return HSCAL_ERR_LOW_MEMORY;
   }

   rc = hscal_pzgeqrf_my_tcomp_tcomm
        (
          1,
          myrow, mycol,
          M, N,
          IA, JA, DESCA,
          nprow, npcol,
          NULL, tcomm_l
   );

   if (rc != HSCAL_SUCCESS)
   {
      return rc;
   }

   for (i = 0; i < nprow; i++)
   {
      for (j = 0; j < npcol; j++)
      {
          tcomm[HSCALRIX(i, j, nprow, npcol)] 
          = 
          TCOMML(myrow, mycol, i, j, nprow, npcol);
      }
   }

   free(tcomm_l);

   return HSCAL_SUCCESS;
}

/*------------------------------------------------------------------*/

int hscal_pzgeqrf_my_tcomm_to_ij(
   int myrow,
   int mycol,
   int ireceiver,
   int jreceiver,
   int M,
   int N,
   int IA,
   int JA,
   int DESCA[DLEN1_],
   int nprow,
   int npcol,
   int *tcomm,
   int *tcomm_i
)
{
   int i, j, rc;

   if (TCOMMI(myrow, mycol, nprow, npcol)
       != HSCAL_COMM_UNINITIALIZED)
   {
      return TCOMM(myrow, mycol, ireceiver, jreceiver, nprow, npcol);
   }

   TCOMMI(myrow, mycol, nprow, npcol) = 1;

   for (i = 0; i < nprow; i++)
   {
      for (j = 0; j < npcol; j++)
      {
          TCOMM(myrow, mycol, i, j, nprow, npcol) = 0;
      }
   }

   rc = hscal_pzgeqrf_my_tcomp_tcomm
        (
          1,
          myrow, mycol,
          M, N,
          IA, JA, DESCA,
          nprow, npcol,
          NULL, tcomm
    );

    if (rc != HSCAL_SUCCESS)
    {
       return rc;
    }

    HSCAL_DEBUG(
         __FILE__, __FUNCTION__,__LINE__,
         "(%d, %d) -> (%d, %d), (p, q)=(%d, %d) %d",
         myrow, mycol, ireceiver, jreceiver, nprow, npcol,
         TCOMM(myrow, mycol, ireceiver, jreceiver, nprow, npcol)
    );

    return TCOMM(myrow, mycol, ireceiver, jreceiver, nprow, npcol);
}

/*------------------------------------------------------------------*/

int hscal_pzgeqrf_my_tcomp_tcomm(
   int init_llds_f,
   int myrow,
   int mycol,
   int M,
   int N,
   int IA,
   int JA,
   int DESCA[DLEN1_],
   int nprow,
   int npcol,
   double *tcomp,
   int *tcomm
)
{
    hscalinteger c__1 = 1;
    hscalinteger c__2 = 2;
    hscalinteger c__6 = 6;

    /* System generated locals */
    hscalinteger i__1, i__2, i__3, i__4, i__5;
    hscaldoublereal d__1;
    hscaldoublecomplex z__1;

    /* Local variables */
    hscalinteger i__, j, k;
    hscalinteger jb, jn, mp0, nq0, ipw, idum1[1], idum2[1];
    hscalinteger icoff, iacol, iinfo, iarow, lwmin, ictxt;
    char colbtop[1], rowbtop[1];

    int     *m      = &M;
    int     *n      = &N;
    int     *ia     = &IA;
    int     *ja     = &JA;
    int     *desca  = (int*)DESCA;
    int     info_l, *info = &info_l;

    double  * tcomp_l = tcomp;
    int     * tcomm_l = tcomm;

    /* Parameter adjustments */
    --desca;

    if (init_llds_f == 1)
    {
       HSCAL_PXGEQRF_INIT_LLDS(
            (desca+1),
            myrow, nprow
       );
    }

    HSCAL_DEBUG(
        __FILE__, __FUNCTION__,__LINE__, 
        "PZGEQRF: (%d, %d): (desca+1)[M1_]=%d (desca+1)[MB1_]=%d myrow=%d (desca+1)[RSRC1_]=%d nprow=%d (desca+1)[LLD1_]=%d",
        myrow, mycol, (desca+1)[M1_], (desca+1)[MB1_], myrow, (desca+1)[RSRC1_], nprow, (desca+1)[LLD1_]
    );

/*     Test the input parameters */
    *info = 0;
    if (nprow == -1) {
	*info = -602;
    } else {
	hscal_chk1mat(m, &c__1, n, &c__2, ia, ja, &desca[1], &c__6, info,
                    myrow, mycol, nprow, npcol
      );
	if (*info == 0) {
	    icoff = (*ja - 1) % desca[6];
	    iarow = indxg2p_(ia, &desca[5], &myrow, &desca[7], &nprow);
	    iacol = indxg2p_(ja, &desca[6], &mycol, &desca[8], &npcol);
	    i__1 = *m + (*ia - 1) % desca[5];
	    mp0 = numroc_(&i__1, &desca[5], &myrow, &iarow, &nprow);
	    i__1 = *n + icoff;
	    nq0 = numroc_(&i__1, &desca[6], &mycol, &iacol, &npcol);
	    lwmin = desca[6] * (mp0 + nq0 + desca[6]);

          d__1 = (hscaldoublereal) lwmin;
          z__1.r = d__1, z__1.i = 0.;
	}
        idum1[0] = 1;
	idum2[0] = 9;
	hscal_pchk1mat(
           m, &c__1, n, &c__2, ia, ja, &desca[1], &c__6, &c__1, idum1, 
		idum2, info,
           &myrow, &mycol, &nprow, &npcol
      );
    }

    if (*info != 0) {
	i__1 = -(*info);
       hscal_PB_Cabort(
            "PZGEQRF",
            i__1,
            nprow, npcol, myrow, mycol
       );
       return HSCAL_SUCCESS;
    }

/*     Quick return if possible */

    if (*m == 0 || *n == 0) {
	return HSCAL_SUCCESS;
    }

    k = min(*m,*n);
    ipw = desca[6] * desca[6] + 1;
    pb_topget__(&ictxt, "Broadcast", "Rowwise", rowbtop);
    pb_topget__(&ictxt, "Broadcast", "Columnwise", colbtop);
    pb_topset__(&ictxt, "Broadcast", "Rowwise", "I-ring");
    pb_topset__(&ictxt, "Broadcast", "Columnwise", " ");

/*     Handle the first block of columns separately */

/* Computing MIN */
    i__1 = iceil_(ja, &desca[6]) * desca[6], i__2 = *ja + k - 1;
    jn = min(i__1,i__2);
    jb = jn - *ja + 1;

/*     Compute the QR factorization of the first block A(ia:ia+m-1,ja:jn) */

    hscal_pzgeqr2_my_tcomp_tcomm(
         0,
         myrow, mycol,
         M, jb, IA, JA, &desca[1],
         nprow, npcol,
         tcomp, tcomm
    );

    if (*ja + jb <= *ja + *n - 1) {

/*        Form the triangular factor of the block reflector */
/*        H = H(ja) H(ja+1) . . . H(jn) */

	hscal_pzlarft_my_tcomp_tcomm(
           0,
           myrow, mycol,
           'F', 'C', M, jb, IA, JA, &desca[1], 
           nprow, npcol,
           tcomp, tcomm
      );

/*        Apply H' to A(ia:ia+m-1,ja+jb:ja+n-1) from the left */

	i__1 = *n - jb;
	i__2 = *ja + jb;
	hscal_pzlarfb_my_tcomp_tcomm(
           0,
           myrow, mycol,
           'L', 'C', 'F', 'C', M, i__1, jb, IA, JA, &desca[1], 
           IA, i__2, &desca[1], 
           nprow, npcol,
           tcomp, tcomm
      );
    }

/*     Loop over the remaining blocks of columns */

    i__1 = *ja + k - 1;
    i__2 = desca[6];
    for (j = jn + 1; i__2 < 0 ? j >= i__1 : j <= i__1; j += i__2) {
/* Computing MIN */
	i__3 = k - j + *ja;
	jb = min(i__3,desca[6]);
	i__ = *ia + j - *ja;

/*        Compute the QR factorization of the current block */
/*        A(i:ia+m-1,j:j+jb-1) */

	i__3 = *m - j + *ja;
	hscal_pzgeqr2_my_tcomp_tcomm(
           0,
           myrow, mycol,
           i__3, jb, i__, j, &desca[1], 
           nprow, npcol,
           tcomp, tcomm
      );

	if (j + jb <= *ja + *n - 1) {

/*           Form the triangular factor of the block reflector */
/*           H = H(j) H(j+1) . . . H(j+jb-1) */

	    i__3 = *m - j + *ja;
	    hscal_pzlarft_my_tcomp_tcomm(
               0,
               myrow, mycol,
               'F', 'C', i__3, jb, i__, j, &desca[1],
               nprow, npcol,
               tcomp, tcomm
          );

/*           Apply H' to A(i:ia+m-1,j+jb:ja+n-1) from the left */

	    i__3 = *m - j + *ja;
	    i__4 = *n - j - jb + *ja;
	    i__5 = j + jb;
	    hscal_pzlarfb_my_tcomp_tcomm(
               0,
               myrow, mycol,
               'L', 'C', 'F', 'C', i__3, i__4, jb, i__, j, &desca[1],
		   i__, i__5, &desca[1],
               nprow, npcol,
               tcomp, tcomm
          );
	}

/* L10: */
    }

    pb_topset__(&ictxt, "Broadcast", "Rowwise", rowbtop);
    pb_topset__(&ictxt, "Broadcast", "Columnwise", colbtop);

   return HSCAL_SUCCESS;
}

/*------------------------------------------------------------------*/

int hscal_pzgeqrf_tcomp_tcomm(
   int M,
   int N,
   int IA,
   int JA,
   int DESCA[DLEN1_],
   int nprow,
   int npcol,
   double *tcomp,
   int *tcomm
)
{
   int i, rc, myrow, mycol;

   for (i = 0; i < nprow*npcol; i++)
   {
      tcomp[i] = 0;
   }

   for (i = 0; i < nprow*npcol*nprow*npcol; i++)
   {
      tcomm[i] = 0;
   }

   for (myrow = 0; myrow < nprow; myrow++)
   {
      for (mycol = 0; mycol < npcol; mycol++)
      {
         rc = hscal_pzgeqrf_my_tcomp_tcomm
              (
                1,
                myrow, mycol,
                M, N,
                IA, JA, DESCA,
                nprow, npcol,
                tcomp, tcomm
         );

         if (rc != HSCAL_SUCCESS)
         {
            return rc;
         }
      }
   }

   return HSCAL_SUCCESS;
}

/*------------------------------------------------------------------*/

int hscal_pzgeqrf_info(
   int * M,
   int * N,
   int * IA,
   int * JA,
   int * DESCA,
   int   nprow,
   int   npcol,
   double * tcomp,
   int    * tcomm
)
{
   int i, rc, myrow, mycol;

   for (i = 0; i < nprow*npcol; i++)
   {
      tcomp[i] = 0;
   }

   for (i = 0; i < nprow*npcol*nprow*npcol; i++)
   {
      tcomm[i] = 0;
   }

   for (myrow = 0; myrow < nprow; myrow++)
   {
      for (mycol = 0; mycol < npcol; mycol++)
      {
         rc = hscal_pzgeqrf_my_tcomp_tcomm
              (
                1,
                myrow, mycol,
                *M, *N,
                *IA, *JA, DESCA,
                nprow, npcol,
                tcomp, tcomm
         );

         if (rc != HSCAL_SUCCESS)
         {
            return rc;
         }
      }
   }

   return HSCAL_SUCCESS;
}

/*------------------------------------------------------------------*/

int hscal_pzgeqrf_info_i(
   int * M,
   int * N,
   int * IA,
   int * JA,
   int * DESCA,
   int   nprow,
   int   npcol,
   double * tcomp,
   int    * tcomm
)
{
   int i, rc, myrow, mycol;

   for (myrow = 0; myrow < nprow; myrow++)
   {
      for (mycol = 0; mycol < npcol; mycol++)
      {
         rc = hscal_pzgeqrf_my_tcomp_tcomm
              (
                1,
                myrow, mycol,
                *M, *N,
                *IA, *JA, DESCA,
                nprow, npcol,
                tcomp, tcomm
         );

         if (rc != HSCAL_SUCCESS)
         {
            return rc;
         }
      }
   }

   return HSCAL_SUCCESS;
}

/*------------------------------------------------------------------*/
